function [Sest,iter]=cosamp_(Phi,u,K,tol1,tol2)

%COSAMP edited by Daryl K Lim - 

%modified par P.Dymarski - only the exit conditions
 
% K : sparsity of Sest    -   i.e. number of searched atoms 
% Phi : obs matrix        -   i.e. dictionary
% u: obs vector           -   i.e. vector x being modeled

% tol1 - stopping condition if u almost perfectly modeled (x = x_hat) 
% tol2 - stopping condition if gains (Sest) almost the same in 2 consecutive iterations
% iter - number of iterations


% Written by David Mary
% This script/program is released under the Commons Creative Licence
% with Attribution Non-commercial Share Alike (by-nc-sa)
% http://creativecommons.org/licenses/by-nc-sa/3.0/
% Disclaimer: the short answer, this script is for educational purpose only.
% More Disclaimer:  http://igorcarron.googlepages.com/disclaimer


% Init.
[siz1,siz2]=size(Phi); 

Sest=zeros(siz2,1);
Sest1=ones(siz2,1); % % previous gains,  ~= Sest

n2old=10^10;
maxiterations=100;

Sest_opt=Sest;
n2min=n2old;
  
v=u;
t=1; T2=[];
while t <= maxiterations 
   [k,z]=sort(abs(Phi'*v));
   k=flipud(k); z=flipud(z);
Omega=z(1:2*K);
T=sort(union(Omega,T2));phit=Phi(:,T);
%keyboard
b=pinv(phit)*u;                     
[k3,z3]=sort(abs(b));
k3=flipud(k3); z3=flipud(z3);
Sest=zeros(siz2,1);
Sest(T(z3(1:K))) = b(z3(1:K));
[k2,z2]=sort(abs(Sest));
k2=flipud(k2); z2=flipud(z2);
T2=z2(1:K);
v=u-Phi*Sest;
d=0;n2=norm(v); 
iter=t;



if n2 < tol1   %  x (here: u) perfectly modeled 
    d=1; t=1e10;
%    disp('CoSaMP: success');
end
if norm(Sest-Sest1) < tol2   % the same gains in 2 consecutive iterations  
    d=1; t=1e10;
    Sest=Sest_opt;
%    disp('CoSaMP: stable point attained');
end

if norm(Sest-Sest_opt) < tol2   % the same gains in 2 iterations    
    d=1; t=1e10;
    Sest=Sest_opt;
%    disp('CoSaMP: possible oscillations');
end

%if  n2 > n2old   % error grows, but here no stop (it may diminish in the next iterations)
%    Sest=Sest1;
%    d=1; t=1e10;
%    disp('CoSaMP: unstable');
%end

if n2 < n2min
  n2min=n2;
  Sest_opt=Sest;
end

 t=t+1;
 Sest1=Sest;
 n2old=n2;

end

Sest=Sest_opt;
n2=n2min;

% if d==0
%     disp('CoSaMP: failed')
% end
 