function [Sest,iter]=sp_(Phi,u,K,tol1,tol2)

%version Sturm
%modified par P.Dymarski - only the stopping conditions

% subspace pursuit
% K : sparsity of Sest    -   i.e. number of searched atoms 
% Phi : obs matrix        -   i.e. dictionary
% u: obs vector           -   i.e. vector x being modeled

% tol1 - stopping condition if u almost perfectly modeled (x = x_hat) 
% tol2 - stopping condition if gains (Sest) almost the same in 2 consecutive iterations
% iter - number of iterations


% Init.
[siz1,siz2]=size(Phi); 

Sest=zeros(siz2,1);
Sest1=ones(siz2,1); % previous gains, ~= Sest
n2old=10^10;

Sest_opt=Sest;
n2min=n2old;

maxiterations=100;
numericalprecision = 1e-12;
  
v=u;
t=1; T2=[];
while t <= maxiterations

    y = abs(Phi'*v);

   [vals,z]=sort(y);
   vals=flipud(vals); z=flipud(z);

% Omega=z(1:K);
Omega = find(y >= vals(K) & y > numericalprecision);

T=sort(union(Omega,T2));  
%T = union(Omega,T2);         % Sturm version - no sort???


phit=Phi(:,T);

%b=pinv(phit)*u;    
b = abs(pinv(phit)*u);

                 
[k3,z3]=sort(b);
k3=flipud(k3); z3=flipud(z3);
vals=k3;

Sest=zeros(siz2,1);
%Sest(T(z3(1:K))) = b(z3(1:K));

Sest(T(find(b >= vals(K) & b > numericalprecision))) = ...
    b(find(b >= vals(K) & b > numericalprecision));

[k2,z2]=sort(abs(Sest));
%[k2,z2]=sort(Sest); %      %  Sturm version -  no abs??



k2=flipud(k2); z2=flipud(z2);

Told = T2;

T2=z2(1:K);

  phit = Phi(:,T2);
  b = pinv(phit)*u;
  Sest = zeros(siz2,1);
  Sest(T2) = b;
  v = u - Phi(:,T2)*b;



%v=u-Phi*Sest;


d=0;n2=norm(v); 
iter=t;
if n2 < tol1   % x (here: u) perfectly modeled 

    d=1; t=1e10;
%    disp('SP: success');
end
if norm(Sest-Sest1) < tol2    % the same gains in 2 consecutive iterations 

    d=1; t=1e10;
    Sest=Sest_opt;

%    disp('CoSaMP: stable point attained');
end

if norm(Sest-Sest_opt) < tol2   % the same gains in 2 iterations 
    d=1; t=1e10;
    Sest=Sest_opt;
%    disp('CoSaMP: possible oscillations');
end

%if  n2 > n2old   % error grows, but here no stop (it may diminish in the next iterations)

%    Sest=Sest1;
%    d=1; t=1e10;
%    disp('SP: unstable');
%end

if n2 < n2min
  n2min=n2;
  Sest_opt=Sest;
end


 t=t+1;
 Sest1=Sest;
 n2old=n2;

end

Sest=Sest_opt;
n2=n2min;

% if d==0
%     disp('SP: failed')
% end
 